SystemOrganization addCategory: #Project!

WAComponent subclass: #System
	instanceVariableNames: 'TopicTable TopicToBeAproved UserTable MaterialTable'
	classVariableNames: 'Admin ContentId DidaticStructureId Id Single UserId'
	poolDictionaries: ''
	category: 'Project'!

!System commentStamp: 'RFG 5/2/2013 13:21' prior: 0!
The main class representing the own system.!

!System class methodsFor: 'rendering' stamp: 'RicardoGuimaraes 6/30/2013 16:40'!
canBeRoot
	^true.! !

!System class methodsFor: 'initialization' stamp: 'RicardoGuimaraes 7/1/2013 15:30'!
create
	"Creates an empty system."
	| userManager|
	Single isNil
		ifTrue: [
			Single := self new.
			Single TopicTable: Dictionary new.
			Single UserTable: Dictionary new.
			Single MaterialTable: Dictionary new.
			Single TopicToBeAproved:  OrderedCollection new.
			userManager := UserManager new.
			userManager registerDefaultAdmin.
			].
		^Single.! !

!System class methodsFor: 'initialization' stamp: 'RFG 5/13/2013 15:11'!
reset
	"Re-creates an empty system."
	
	| newSys |
	Single := nil.
	newSys := System create.
	^newSys.
	
	! !

!System methodsFor: 'accessing' stamp: 'RicardoGuimaraes 7/1/2013 11:52'!
Admin
	"Returns the main admin of the system (created at system start)."

	^Admin.! !

!System methodsFor: 'accessing' stamp: 'RicardoGuimaraes 7/1/2013 11:52'!
Admin: anUser
	"Sets the main admin of the system (created at system start)."

	^Admin := anUser.! !

!System methodsFor: 'accessing' stamp: 'RicardoGuimaraes 7/1/2013 11:14'!
MaterialTable
	"Returns the dictionary with all DidaticMaterial."

	^MaterialTable.! !

!System methodsFor: 'accessing' stamp: 'RicardoGuimaraes 7/1/2013 11:14'!
MaterialTable: aDictionary
	"Sets the dictionary with all material."

	MaterialTable :=  aDictionary! !

!System methodsFor: 'accessing' stamp: 'RFG 5/3/2013 20:54'!
TopicTable
	"Returns the dictionary with all topics."

	^TopicTable.! !

!System methodsFor: 'accessing' stamp: 'RFG 5/3/2013 20:54'!
TopicTable: aDictionary
	"Sets the dictionary with all topics."

	TopicTable :=  aDictionary! !

!System methodsFor: 'accessing' stamp: 'RFG 6/14/2013 19:53'!
TopicToBeAproved

	^ TopicToBeAproved! !

!System methodsFor: 'accessing' stamp: 'RFG 5/13/2013 15:05'!
TopicToBeAproved: aCollection
	"Sets the orderedCollection with all topics that aren't aproved yet."

	^TopicToBeAproved :=  aCollection! !

!System methodsFor: 'accessing' stamp: 'RFG 5/3/2013 20:17'!
UserTable
	"Returns the dictionary with all users."

	^UserTable.! !

!System methodsFor: 'accessing' stamp: 'RFG 5/3/2013 20:13'!
UserTable: aDictionary
	"Sets the system user table."

	UserTable := aDictionary.! !

!System methodsFor: 'adding' stamp: 'RFG 5/13/2013 13:03'!
addTopic: aTopic

	TopicTable at: aTopic id put: aTopic.
	TopicToBeAproved addLast: aTopic id.! !

!System methodsFor: 'adding' stamp: 'RicardoGuimaraes 6/28/2013 10:05'!
addUser: anUser

	UserTable at: anUser id put: anUser.! !

!System methodsFor: 'rendering' stamp: 'RicardoGuimaraes 6/30/2013 17:23'!
children
	| sys |
	sys := System create.
	^sys UserTable values.! !

!System methodsFor: 'accessing' stamp: 'RFG 5/13/2013 13:53'!
findCourse: anId
	"Returns a course with the given ID."
	
	| anArray
	  aTopic
	  aCourse
	  index   |
	
	index := 1.
	anArray := TopicTable associations.
	[index <= anArray size]
	whileTrue: [
		aTopic := anArray at: index.
		aCourse := aTopic value findCourse: anId.
		aCourse notNil
		ifTrue:[
			^aCourse
			].
		index := index +1
		].
		^(KeyNotFound key: anId) signal.! !

!System methodsFor: 'accessing' stamp: 'AP 6/9/2013 15:26'!
findGroup: anId
	"Returns a group with the given ID."
	
	| isThere 
	 group   |
	TopicTable do:  [:topic |
		topic subjectTable do: [:subject |
			isThere := subject groupTable includesKey: anId.
			isThere ifTrue:[
				group := subject groupTable at: anId]
		].
	].
	group isNil 
		ifTrue:[^(KeyNotFound key: anId) signal.]
		ifFalse:[^group].
	! !

!System methodsFor: 'accessing' stamp: 'RFG 5/13/2013 21:47'!
findLecture: anId
	"Returns a lecture with the given ID."
	
	|isThere  
	lecture   |
	TopicTable do:  [:topic |
		topic subjectTable do: [:subject |
			isThere := subject lectureTable includesKey: anId.
			isThere ifTrue:[
				 lecture := subject lectureTable at: anId]
		].
	].
	 lecture isNil 
		ifTrue:[^(KeyNotFound key: anId) signal.]
		ifFalse:[^ lecture].
	! !

!System methodsFor: 'accessing' stamp: 'RFG 5/13/2013 14:13'!
findSubject: anId
	"Returns a subject with the given ID."
	
	| anArray
	  aTopic
	  aSubject
	  index     |
	
	index := 1.
	anArray := TopicTable associations.
	[index <= anArray size]
	whileTrue: [
		aTopic := anArray at: index.
		aSubject := aTopic value findSubject: anId.
		aSubject notNil
		ifTrue:[
			^aSubject
			].
		index := index +1
		].
		^(KeyNotFound key: anId) signal.! !

!System methodsFor: 'accessing' stamp: 'AP 5/12/2013 21:13'!
findTopic: anId

	^TopicTable at: anId! !

!System methodsFor: 'accessing' stamp: 'AP 5/12/2013 21:13'!
findUser: anId

	^UserTable at: anId! !

!System methodsFor: 'id management' stamp: 'RicardoGuimaraes 6/28/2013 20:47'!
nextContentId
	"Returns the next content id avaliable."

	ContentId isNil
		ifTrue:[ContentId := 0]
		ifFalse:[ContentId := ContentId + 1].
	^ContentId.! !

!System methodsFor: 'id management' stamp: 'RicardoGuimaraes 6/28/2013 20:47'!
nextDidaticStructureId
	"Returns the next didatic structure id avaliable."

	DidaticStructureId isNil
		ifTrue:[DidaticStructureId := 0]
		ifFalse:[DidaticStructureId := DidaticStructureId + 1].
	^DidaticStructureId.! !

!System methodsFor: 'id management' stamp: 'RicardoGuimaraes 6/28/2013 20:47'!
nextId
	"Returns the next id avaliable."

	Id isNil
		ifTrue:[Id := 0]
		ifFalse:[Id := Id + 1].
	^Id.! !

!System methodsFor: 'id management' stamp: 'RicardoGuimaraes 6/28/2013 20:47'!
nextUserId
	"Returns the next user id avaliable."

	UserId isNil
		ifTrue:[UserId := 0]
		ifFalse:[UserId := UserId + 1].
	^UserId.! !

!System methodsFor: 'removing' stamp: 'RicardoGuimaraes 6/28/2013 10:06'!
removeTopic: aTopic
	"Remove a Topic from the System"
	TopicTable removeKey: aTopic id.! !

!System methodsFor: 'removing' stamp: 'RicardoGuimaraes 6/28/2013 10:06'!
removeUser: anUser
	"Remove an user from the System"
	UserTable removeKey: anUser id.! !

!System methodsFor: 'rendering' stamp: 'RicardoGuimaraes 6/30/2013 18:29'!
renderContentOn: html
	| sys |
	sys := System create.
	html heading: 'System'.	
	sys UserTable do: [ :each | html render: each ] separatedBy: [ html horizontalRule ].
	! !

!System methodsFor: 'accessing' stamp: 'AP 5/11/2013 21:53'!
topicToModerator
	"Gives a topic for the moderator analysis"
	^TopicToBeAproved at: 1! !

User subclass: #Moderator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Moderator commentStamp: 'RFG 5/11/2013 12:24' prior: 0!
Represents a topic moderator in the system.!

Moderator subclass: #Administrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Administrator commentStamp: 'RFG 5/11/2013 14:52' prior: 0!
Represents special users who can create topics and erase/create anything.!

!Administrator methodsFor: 'publishing' stamp: 'RicardoGuimaraes 6/29/2013 10:45'!
aproveTopic
	"Make a topic visible for the public. In other words, the administrator aproves a topic
	using this function"
	| anId
	  aTopic
	  sys	|
	
	sys := System create.
	anId := sys topicToModerator.
	aTopic := sys findTopic: anId.
	aTopic visible: true.
! !

!Administrator methodsFor: 'creation' stamp: 'RicardoGuimaraes 6/29/2013 09:15'!
createTopic: aName by: anUser

	| sys
	  aTopic |
	
	aTopic := Topic name: aName creator: anUser.
	sys := System create.
	sys addTopic: aTopic.! !

!Administrator methodsFor: 'creation' stamp: 'RicardoGuimaraes 6/29/2013 09:16'!
createUser: aName
	
	| newUser 
	  sys        |
	
	newUser := User newNamed: aName.
	sys := System create.
	sys addUser: newUser.! !

!Administrator methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/29/2013 11:09'!
delete: aDidaticStructure
	"Deletes a didatic structure."

	aDidaticStructure delete.! !

!Administrator methodsFor: 'nominate' stamp: 'RicardoGuimaraes 6/29/2013 14:00'!
nominateAdministrator: anUser
	"Give moderator privileges to an user"

	| newAdministrator |
	
	newAdministrator := Administrator newNamed: anUser name withId: anUser id.
	anUser become: newAdministrator.
	"newAdministrator courseList: anUser courseList. 
	newAdministrator groupList: anUser groupList.
	newAdministrator contentList: anUser contentList.
	
	sys := System create.
	sys removeUser: anUser id.
	sys addUser: newAdministrator id."
	
	^anUser.! !

!Administrator methodsFor: 'nominate' stamp: 'RicardoGuimaraes 6/29/2013 14:00'!
nominateModerator: anUser onTopic: aTopic
	"Give moderator privileges to an user"

	| newModerator |
	
	newModerator := Moderator newNamed: anUser name withId: anUser id.
	anUser become: newModerator.
	"newModerator courseList: anUser courseList. 
	newModerator groupList: anUser groupList.
	newModerator contentList: anUser contentList.
	
	sys := System create.
	sys removeUser: anUser id.
	sys addUser: newModerator.
	
	aTopic := sys findTopic: aTopicId."
	aTopic addModerator: anUser .
	
	^anUser.! !

!Moderator methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/29/2013 11:07'!
delete: aDidaticStructure
	"Tries to delete a Didatic Structure "

	| isModerator |
	
	isModerator := aDidaticStructure topic moderatorList includes: self.
	isModerator
		ifTrue:[aDidaticStructure delete.]
		ifFalse:[super delete: aDidaticStructure.]! !

!Moderator methodsFor: 'publishing' stamp: 'RicardoGuimaraes 6/28/2013 10:55'!
hide: aDidaticStructure
	"Turns a didatic structure invisible to all commom users non-owners."

	| sys
	  mod     |
	
	sys := SystemFacade new.
	mod := sys moderator: self changeVisibilityOf: aDidaticStructure.
	
	mod isNil
		ifFalse: [
			aDidaticStructure visible: false.
			].! !

!Moderator methodsFor: 'publishing' stamp: 'AP 6/9/2013 16:27'!
publish: aDidaticStructure
	"Turns a didatic structure invisible to all commom users non-owners."

	| sys
	  mod     |
	
	sys := SystemFacade new.
	mod := sys moderator: self changeVisibilityOf: aDidaticStructure.
	
	mod isNil
		ifFalse: [
			aDidaticStructure visible: true.
			].! !

Object subclass: #DidaticStructure
	instanceVariableNames: 'id name creator ownerList visible topic'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!DidaticStructure commentStamp: 'RFG 5/11/2013 13:53' prior: 0!
Mudar o nome dessa classe. It represents: Lecture U Group U Subject U Courses!

DidaticStructure subclass: #Course
	instanceVariableNames: 'subjectList userList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Course commentStamp: 'RFG 5/2/2013 11:43' prior: 0!
Represents a course in the system.!

!Course class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/29/2013 08:01'!
name: aName by: anUser topic: aTopic
	"Creates a new Course named aString created  by anUser in aTopic."
	
	| newCourse |
	
	newCourse := super name: aName creator:  anUser topic: aTopic.
	
	newCourse userList isNil
		ifTrue: [newCourse userList: OrderedCollection new].
		
	newCourse subjectList isNil
		ifTrue: [newCourse subjectList: OrderedCollection new].
		
	aTopic addCourse: newCourse.
	newCourse name: newCourse  name, 'torrada'.
	^newCourse! !

!Course class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/29/2013 08:03'!
name: aName creator: anUser topic: aTopic
	"Creates a new Course named aString created  by anUser in aTopic."
	
	| newCourse |
	
	newCourse := super name: aName creator:  anUser topic: aTopic.
	
	newCourse userList isNil
		ifTrue: [newCourse userList: OrderedCollection new].
		
	newCourse subjectList isNil
		ifTrue: [newCourse subjectList: OrderedCollection new].
		
	aTopic addCourse: newCourse.
	newCourse name: newCourse  name.
	^newCourse! !

!Course methodsFor: 'inclusion' stamp: 'RicardoGuimaraes 6/28/2013 19:40'!
addSubject: anSubject
	"Adds a subject into the course."

	subjectList add: anSubject.! !

!Course methodsFor: 'inclusion' stamp: 'RicardoGuimaraes 6/28/2013 10:42'!
addUser: anUser
	"Adds a user into the course."

	userList add: anUser.! !

!Course methodsFor: 'deletion' stamp: 'RicardoGuimaraes 6/28/2013 19:50'!
delete
	"Removes the course from the System."
	
	userList do:[:user | user withdrawCourse: self.].
	
	topic courseTable removeAt: self id.! !

!Course methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/28/2013 19:50'!
removeSubject: anSubject.
	"Removes the subject"
	
	subjectList remove: anSubject.! !

!Course methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/28/2013 19:52'!
removeUser: anUser
	"Removes the user with the given User."
	
	userList remove: anUser.! !

!Course methodsFor: 'accessing' stamp: 'RFG 5/11/2013 09:58'!
subjectList
	"Returns the list of subjects included in the course."

	^subjectList.! !

!Course methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/28/2013 11:08'!
subjectList: aListOfSubjects
	"Sets the list of subjects included in the course."

	subjectList := aListOfSubjects.! !

!Course methodsFor: 'accessing' stamp: 'RFG 5/11/2013 10:36'!
userList
	"Returns the list of inscribed users."

	^userList.! !

!Course methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/28/2013 11:08'!
userList: aListOfUsers
	"Sets the list of inscribed users."

	userList := aListOfUsers.! !

!DidaticStructure class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/28/2013 20:53'!
name: aName creator: anUser topic: aTopic
	"Creates a new DidaticStructure named aString by an user on the topic."

	| newDidaticStructure 
	  sys 				 	|
	
	newDidaticStructure := self new.
	sys := System create.
	newDidaticStructure id: sys nextDidaticStructureId.
	newDidaticStructure name: aName.
	newDidaticStructure creator: anUser.
	newDidaticStructure topic: aTopic.
	newDidaticStructure visible: False. 
	
	newDidaticStructure ownerList isNil
		ifTrue: [newDidaticStructure ownerList: OrderedCollection new].
	
	^newDidaticStructure.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/11/2013 08:03'!
creator
	"Returns the creator."

	^creator.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/28/2013 19:29'!
creator: anUser
	"Sets the creator."

	creator := anUser.! !

!DidaticStructure methodsFor: 'deletion' stamp: 'RFG 5/13/2013 19:08'!
delete
	"Removes the didatic structure from the system."

	^self subclassResponsibility.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/13/2013 12:22'!
id
	"Returns the id."

	^id.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/13/2013 12:22'!
id: anID
	"Sets the id."
	
	id := anID.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/10/2013 18:13'!
name
	"Returns the name."

	^name.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/10/2013 18:14'!
name: aString
	"Sets the name."

	name := aString.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/11/2013 08:07'!
ownerList
	"Returns the list of owners."

	^ownerList.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/28/2013 19:39'!
ownerList: aListOfUsers
	"Sets the list of owners."

	ownerList := aListOfUsers.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/11/2013 14:26'!
topic
	"Returns the topic."

	^topic.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/11/2013 14:26'!
topic: aTopic
	"Sets the topic."

	topic := aTopic.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/11/2013 10:01'!
visible
	"Returns whether the structure is visible or not by non-owners users"

	^visible.! !

!DidaticStructure methodsFor: 'accessing' stamp: 'RFG 5/11/2013 10:03'!
visible: boolean
	"Sets whether the structure is visible or not by non-owners users"

	visible := boolean.! !

DidaticStructure subclass: #Group
	instanceVariableNames: 'userList instructor subject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Group commentStamp: 'RFG 5/9/2013 13:46' prior: 0!
Represents a group in the system. Set of students at inscribed in a subject with a instructor.!

!Group class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/21/2013 21:24'!
name: aName creator: anUser subject:  anSubject  instructor: anInstructor
	"Creates a new Group in a Subject and with a instructor by the given user"
	
	| newGroup |
	
	newGroup := super name: aName creator: anUser topic: anSubject topic.
	newGroup instructor: anInstructor.
	newGroup subject: anSubject.
	
	newGroup userList isNil
		ifTrue: [newGroup userList: OrderedCollection new].
	anSubject addGroup: newGroup.
		
	^newGroup.
	! !

!Group methodsFor: 'adding' stamp: 'RicardoGuimaraes 6/28/2013 19:52'!
addUser: anUser
	"Adds a user into the group."

	userList add: anUser.! !

!Group methodsFor: 'deletion' stamp: 'RicardoGuimaraes 6/29/2013 08:43'!
delete
	"Removes a group from the system."
	
	userList do:[:user | user leaveGroup: self.].
	subject groupTable removeKey: (self id).! !

!Group methodsFor: 'accessing' stamp: 'RFG 5/11/2013 11:10'!
instructor
	"Returns the group's instructor."

	^instructor.! !

!Group methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/29/2013 08:49'!
instructor: anUser
	"Sets the group's instructor."

	instructor := anUser.! !

!Group methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/29/2013 08:58'!
removeUser: anUser
	"Removes the user from the Group."
	
	| sysFacade |
	
	sysFacade := SystemFacade new.
	userList remove: anUser.
	sysFacade  removeUser: anUser fromGroup: self.
	! !

!Group methodsFor: 'accessing' stamp: 'RFG 5/11/2013 11:10'!
subject
	"Returns the group's subject."

	^subject.! !

!Group methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/29/2013 08:58'!
subject: aSubject
	"Sets the group's subject."

	subject := aSubject.! !

!Group methodsFor: 'accessing' stamp: 'RFG 5/11/2013 10:54'!
userList
	"Returns the list of inscribed users."

	^userList.! !

!Group methodsFor: 'accessing' stamp: 'AP 5/12/2013 21:09'!
userList: aListOfIds
	"Sets the list of inscribed users."

	userList := aListOfIds.! !

DidaticStructure subclass: #Lecture
	instanceVariableNames: 'subject userList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Lecture commentStamp: 'RFG 5/10/2013 18:02' prior: 0!
Represents a Lecture of a Subject!

!Lecture class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/21/2013 21:34'!
name: aName creator: anUser  subject:  aSubject
	"Creates a new Lecture in a Subject by an User"

	| newLecture |
	
	newLecture := super name:  aName creator: anUser topic:  aSubject topic.
	newLecture subject: aSubject.
	
	newLecture userList isNil
		ifTrue: [newLecture userList: OrderedCollection new].
		
	aSubject addLecture: newLecture.
	
	^newLecture.! !

!Lecture methodsFor: 'deletion' stamp: 'RicardoGuimaraes 6/29/2013 08:59'!
delete
	"Removes a lecture from the system."

	subject lectureTable removeKey: (self id).! !

!Lecture methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/29/2013 08:59'!
removeUser: anUser.
	"Removes the user with the given ID."
	
	userList remove: anUser.! !

!Lecture methodsFor: 'accessing' stamp: 'RFG 5/11/2013 11:43'!
subject
	"Returns the lecture's subject."

	^subject.! !

!Lecture methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/29/2013 08:59'!
subject: aSubject
	"Sets the lecture's subject."

	subject := aSubject.! !

!Lecture methodsFor: 'accessing' stamp: 'RFG 5/11/2013 11:43'!
userList
	"Returns the list of inscribed users."

	^userList.! !

!Lecture methodsFor: 'accessing' stamp: 'AP 5/12/2013 21:10'!
userList: aListOfIds
	"Sets the list of inscribed users."

	userList := aListOfIds.! !

DidaticStructure subclass: #Subject
	instanceVariableNames: 'groupTable lectureTable responsible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Subject commentStamp: 'RFG 5/6/2013 15:55' prior: 0!
Represents a Subject in the system.!

!Subject class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/21/2013 21:44'!
name: aName creator: anUser topic: aTopic
	"Creates a new Subject managed and created by an User in a Topic. "

	| newSubject |
	
	newSubject := super name: aName creator: anUser topic: aTopic.
	newSubject responsible: anUser.
	
	newSubject groupTable isNil
		ifTrue: [newSubject groupTable: Dictionary new].
	
	newSubject lectureTable isNil
		ifTrue: [newSubject lectureTable: Dictionary new].
	
	aTopic addSubject: newSubject.
	
	^newSubject.! !

!Subject class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/21/2013 21:43'!
name: aName creator: anUser topic: aTopic responsible: responsible
	"Creates a new Subject managed by a teacher responsible in a Topic by an User. "

	| newSubject |
	
	newSubject := super name: aName creator: anUser topic: aTopic.
	newSubject responsible: responsible.
	
	newSubject groupTable isNil
		ifTrue: [newSubject groupTable: Dictionary new].
	
	newSubject lectureTable isNil
		ifTrue: [newSubject lectureTable: Dictionary new].
	
	aTopic addSubject: newSubject.
	
	^newSubject.! !

!Subject methodsFor: 'adding' stamp: 'RFG 5/13/2013 14:25'!
addGroup: aGroup
	"Adds the group to the subject list."

	groupTable at: aGroup id put: aGroup.! !

!Subject methodsFor: 'adding' stamp: 'RFG 5/13/2013 14:28'!
addLecture: aLecture
	"Adds the lecture to the subject list."

	groupTable at: aLecture id put: aLecture.! !

!Subject methodsFor: 'deletion' stamp: 'RFG 5/13/2013 19:51'!
delete
	"Removes the subject from the system."
	
	| system
	  topic    |
	
	groupTable do: [:group | group delete.].
	lectureTable do: [:lecture| lecture delete].
	
	system := System create.
	topic := system findTopic: (self topic).
	topic subjectTable removeAt: (self id).! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/13/2013 19:03'!
findGroup: anGroupId
	"Returns a group at id if present."

	^(groupTable at: anGroupId).! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/13/2013 19:04'!
findLecture: anLectureId
	"Returns a lecture at if present."

	^(lectureTable at: anLectureId).! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/12/2013 09:13'!
groupTable
	"Returns the table of groups that study this discipline."

	^groupTable.! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/12/2013 09:13'!
groupTable: aDict
	"Sets the table of groups that study this discipline."

	groupTable := aDict.! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/13/2013 14:30'!
lectureTable
	"Returns the table of lectures that of this discipline."

	^lectureTable.! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/13/2013 14:29'!
lectureTable: aDict
	"Sets the table of lectures of this discipline."

	lectureTable := aDict.! !

!Subject methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/29/2013 09:05'!
removeGroup: aGroup
	"Removes the group from the group table"
	
	aGroup delete.! !

!Subject methodsFor: 'remotion' stamp: 'RicardoGuimaraes 6/29/2013 09:05'!
removeLecture: aLecture
	"Removes the lecture from the lecture table."
	
	aLecture delete.! !

!Subject methodsFor: 'accessing' stamp: 'RFG 5/12/2013 09:14'!
responsible
	"Returns the teacher responsible for this subject."

	^responsible.! !

!Subject methodsFor: 'accessing' stamp: 'RicardoGuimaraes 6/29/2013 09:05'!
responsible: anUser
	"Sets the teacher responsible for this subject."

	responsible := anUser.! !

DidaticStructure subclass: #Topic
	instanceVariableNames: 'courseTable subjectTable moderatorList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!Topic commentStamp: 'RFG 5/2/2013 13:44' prior: 0!
Represents a topic in the system.!

!Topic class methodsFor: 'instance creation' stamp: 'RicardoGuimaraes 6/29/2013 07:42'!
name: aName creator: anUser
	"Creates a new topic by an User."
	
	|newTopic
	 sys		|
	
	newTopic := super name: aName creator: anUser topic: nil.
	newTopic topic: newTopic.
	
	newTopic courseTable isNil
		ifTrue: [newTopic courseTable: Dictionary new].
		
	newTopic subjectTable isNil
		ifTrue: [newTopic subjectTable: Dictionary new].
	
	newTopic moderatorList isNil
		ifTrue: [newTopic moderatorList: OrderedCollection new].
	newTopic topic: self.	
	sys := System create.
	sys addTopic: newTopic.
	
	^newTopic.! !

!Topic methodsFor: 'adding' stamp: 'AP 5/11/2013 23:34'!
addCourse: aCourse

	courseTable at: aCourse id put: aCourse.

	! !

!Topic methodsFor: 'adding' stamp: 'AP 5/12/2013 20:54'!
addModerator: aModeratorId

	moderatorList add: aModeratorId.! !

!Topic methodsFor: 'adding' stamp: 'RFG 5/13/2013 13:09'!
addSubject: aSubject
	"Adds the subject to the subject list."

	subjectTable at: aSubject id put: aSubject.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 11:16'!
courseTable
	"Returns the dictionary of courses of the topic."

	^courseTable.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 11:17'!
courseTable: aDictionary
	"Sets the dictionary of courses of the topic."

	courseTable := aDictionary.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/5/2013 08:50'!
creator
	"Returns the topic creator."

	^creator.! !

!Topic methodsFor: 'accessing' stamp: 'AP 5/12/2013 21:28'!
creator: anUserId.
	"Sets the topic creator."

	creator := anUserId.! !

!Topic methodsFor: 'deletion' stamp: 'RFG 5/13/2013 19:52'!
delete
	"Removes the topic from the system."
	
	| system
	  topic    |
	
	courseTable do: [:course | course delete.].
	subjectTable do: [:subject| subject delete].
	
	system := System create.
	topic := system findTopic: (self topic).
	system removeTopic: (self id).! !

!Topic methodsFor: 'accessing' stamp: 'AP 5/12/2013 15:25'!
findCourse: aCourseId

	^courseTable at: aCourseId ifAbsent:[^nil].! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/13/2013 14:12'!
findSubject: aSubjectId

	^subjectTable at: aSubjectId ifAbsent:[^nil].! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/13/2013 12:55'!
id
	"Returns the topic id."

	^id.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/13/2013 12:56'!
id: anId
	"Sets the topic id."

	id := anId.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 15:44'!
moderatorList
	"Returns the list of moderators of the topic."

	^moderatorList.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 15:43'!
moderatorList: anOrderedCollection
	"Sets the list of moderators of the topic."

	moderatorList := anOrderedCollection.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 11:15'!
name
	"Returns the topic name."

	^name.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 11:15'!
name: aName.
	"Sets the topic name."

	name := aName.! !

!Topic methodsFor: 'remotion' stamp: 'RFG 5/13/2013 19:43'!
removeCourse: aCourseId
	"Removes the course with the given ID"
	
	| course |
	course := self findGroup: aCourseId.
	course delete.! !

!Topic methodsFor: 'remotion' stamp: 'RFG 5/13/2013 19:45'!
removeSubject: aSubjectId
	"Removes the subject with the given ID"
	
	| subject |
	subject := self findSubject: aSubjectId.
	subject delete.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 15:42'!
subjectTable
	"Returns the dictionary of subjects of the topic."

	^subjectTable.! !

!Topic methodsFor: 'accessing' stamp: 'RFG 5/4/2013 15:41'!
subjectTable: aDictionary
	"Sets the dictionary of subjects of the topic."

	subjectTable := aDictionary.! !

Object subclass: #SystemFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Project'!

!SystemFacade commentStamp: '<historical>' prior: 0!
A SystemFacade is xxxxxxxxx.

Instance Variables
!

!SystemFacade methodsFor: 'deletion' stamp: 'RicardoGuimaraes 6/28/2013 10:15'!
deleteUser: anUser

	"Removes the user from the system."
	| sys |
	
	sys := System create.
	sys  removeUser: anUser.
	
	anUser courseList do: [: course| course removeUser: anUser.].
	anUser groupList do: [: group| group removeUser: anUser.].! !

!SystemFacade methodsFor: 'subscription' stamp: 'RicardoGuimaraes 6/28/2013 10:16'!
enrollUser: anUser atCourse: aCourse
	"Enrolls the user in a course and register this course in the courselist."
	
	aCourse addUser: anUser.! !

!SystemFacade methodsFor: 'subscription' stamp: 'RicardoGuimaraes 6/28/2013 10:41'!
enrollUser: anUser atGroup: aGroup
	"Enrolls the user in a group and register this group in the grouplist."
	
	aGroup addUser: anUser.! !

!SystemFacade methodsFor: 'subscription' stamp: 'RicardoGuimaraes 6/28/2013 10:53'!
makeUser: anUser watchLecture: aLecture
	"Includes the user on the list of user who watched a class."

	aLecture userList add: anUser.! !

!SystemFacade methodsFor: 'moderator' stamp: 'AP 6/9/2013 16:26'!
moderator: aModerator changeVisibilityOf: aDidaticStructure
	"Turns a didatic structure invisible to all commom users non-owners."

	| topic 
	  sys
	  mod     |
	
	sys := System create.
	topic :=  sys findTopic: aDidaticStructure topic.
	
	mod := topic moderatorList at: aModerator id.
	^mod.! !

!SystemFacade methodsFor: 'removing' stamp: 'RicardoGuimaraes 6/28/2013 11:13'!
removeUser: anUser fromCourse: aCourse
	"Remove the user from the Course."
	
	aCourse userList remove: anUser.! !

!SystemFacade methodsFor: 'removing' stamp: 'RicardoGuimaraes 6/28/2013 12:53'!
removeUser: anUser fromGroup: aGroup
	"Makes user leave a group."
	
	aGroup removeUser: anUser.! !

!SystemFacade methodsFor: 'subscription' stamp: 'RicardoGuimaraes 7/1/2013 11:34'!
submitMaterial: submittedFile title: title authors: authorsList description: description type: type licence: licence tags: tags
	| material now user sys|
	sys := System create.
	now := TimeStamp now.
	user := Session user.
	UserManager new canSubmitMaterial ifTrue: [
	material := DidacticMaterial new file: submittedFile; title: title; authors: authorsList; description: description; type: type; licence: licence; tags: tags; creationDate: now; lastUploadDate: now; owner: user.
	material url: (self createMaterialURL: material).
	material id: sys nextContentId.
	sys MaterialTable at: material id put: material.]
	ifFalse: [Error signal: 'User doesn''t have permission to submit material'].! !
